@@ -166,6 +166,7 @@ snmp_mail(struct variable *vp, oid *name, size_t *length,
         return NULL;
 }
 
+static const char global_name[] = "Keepalived";
 static oid global_oid[] = GLOBAL_OID;
 static struct variable8 global_vars[] = {
 	/* version */
@@ -192,17 +193,31 @@ snmp_setup_session_cb(int majorID, int minorID,
 	netsnmp_session *sess = serverarg;
 	if (serverarg == NULL)
 		return 0;
-	/* Because ping are done synchronously, we do everything to
-	   avoid to block too long. Better disconnect from the master
-	   agent than waiting for him... */
+	/*
+	 * Because ping are done synchronously, we do everything to
+	 * avoid to block too long. Better disconnect from the master
+	 * agent than waiting for him...
+	 */
 	sess->timeout = ONE_SEC / 3;
 	sess->retries = 0;
 	return 0;
 }
 
+void snmp_register_mib(oid *myoid, int len, const char *name,
+		       struct variable *variables, int varsize, int varlen)
+{
+	char name_buf[80];
+
+	if (register_mib(name, (struct variable *) variables, varsize,
+			 varlen, myoid, len) != MIB_REGISTERED_OK)
+		log_message(LOG_WARNING, "Unable to register %s MIB", name);
+
+	snprintf(name_buf, sizeof(name_buf), "The MIB module for %s", name);
+	register_sysORTable(myoid, OID_LENGTH(myoid) - 1, name_buf);
+}
+
 void
-snmp_agent_init(oid *myoid, int len, char *name, struct variable *variables,
-		int varsize, int varlen)
+snmp_agent_init(const char *snmp_socket)
 {
 	log_message(LOG_INFO, "Starting SNMP subagent");
 	netsnmp_enable_subagent();
@@ -218,33 +233,38 @@ snmp_agent_init(oid *myoid, int len, char *name, struct variable *variables,
 	    NETSNMP_DS_LIB_DONT_PERSIST_STATE, TRUE);
 	/* Do not load any MIB */
 	setenv("MIBS", "", 1);
-	/* We also register a callback to modify default timeout and
-	   retries value. */
+	/*
+	 * We also register a callback to modify default timeout and
+	 * retries value.
+	 */
 	snmp_register_callback(SNMP_CALLBACK_LIBRARY,
 			       SNMP_CALLBACK_SESSION_INIT,
 			       snmp_setup_session_cb, NULL);
-
-	init_agent(name);
-	/* Ping AgentX less often than every 15 seconds: pinging can
-	   block keepalived. We check every 2 minutes. */
+	/* Specify the socket to master agent, if provided */
+	if (snmp_socket != NULL) {
+		netsnmp_ds_set_string(NETSNMP_DS_APPLICATION_ID,
+				      NETSNMP_DS_AGENT_X_SOCKET,
+				      snmp_socket);
+	}
+	/*
+	 * Ping AgentX less often than every 15 seconds: pinging can
+	 * block keepalived. We check every 2 minutes.
+	 */
 	netsnmp_ds_set_int(NETSNMP_DS_APPLICATION_ID,
 			   NETSNMP_DS_AGENT_AGENTX_PING_INTERVAL, 120);
-	if (register_mib(name, (struct variable *) variables, varsize,
-			 varlen, myoid, len) != MIB_REGISTERED_OK)
-		log_message(LOG_WARNING, "Unable to register MIB");
-	register_mib("Keepalived", (struct variable *) global_vars,
-		     sizeof(struct variable8),
-		     sizeof(global_vars)/sizeof(struct variable8),
-		     global_oid, OID_LENGTH(global_oid));
-	init_snmp(name);
 
-	register_sysORTable(global_oid, OID_LENGTH(global_oid) - 1,
-			    "The MIB module for Keepalived");
+	init_agent(global_name);
+	snmp_register_mib(global_oid, OID_LENGTH(global_oid), global_name,
+			  (struct variable *)global_vars,
+			  sizeof(struct variable8),
+			  sizeof(global_vars)/sizeof(struct variable8));
+	init_snmp(global_name);
 }
 
 void
 snmp_agent_close(oid *myoid, int len, char *name)
 {
 	unregister_sysORTable(myoid, len);
-	snmp_shutdown(name);
+	unregister_sysORTable(global_oid, OID_LENGTH(global_oid));
+	snmp_shutdown(global_name);
 }
