@@ -28,7 +28,6 @@
 #include <net/if_arp.h>
 
 /* local includes */
-#include "vrrp.h"
 #include "vrrp_ipaddress.h"
 
 /* local definitions */
@@ -36,19 +35,19 @@
 #define IPPROTO_ADDR_LEN	4
 
 /* types definition */
-typedef struct _m_arphdr {
-	unsigned short int ar_hrd;	/* Format of hardware address.  */
-	unsigned short int ar_pro;	/* Format of protocol address.  */
-	unsigned char ar_hln;	/* Length of hardware address.  */
-	unsigned char ar_pln;	/* Length of protocol address.  */
-	unsigned short int ar_op;	/* ARP opcode (command).  */
+typedef struct _arphdr {
+	unsigned short int	ar_hrd;	/* Format of hardware address.  */
+	unsigned short int	ar_pro;	/* Format of protocol address.  */
+	unsigned char		ar_hln;	/* Length of hardware address.  */
+	unsigned char		ar_pln;	/* Length of protocol address.  */
+	unsigned short int	ar_op;	/* ARP opcode (command).  */
 
 	/* Ethernet looks like this : This bit is variable sized however...  */
-	unsigned char __ar_sha[ETH_ALEN];	/* Sender hardware address.  */
-	unsigned char __ar_sip[4];	/* Sender IP address.  */
-	unsigned char __ar_tha[ETH_ALEN];	/* Target hardware address.  */
-	unsigned char __ar_tip[4];	/* Target IP address.  */
-} m_arphdr;
+	unsigned char		__ar_sha[ETH_ALEN];	/* Sender hardware address.  */
+	unsigned char		__ar_sip[4];		/* Sender IP address.  */
+	unsigned char		__ar_tha[ETH_ALEN];	/* Target hardware address.  */
+	unsigned char		__ar_tip[4];		/* Target IP address.  */
+} arphdr_t;
 
 /* Global vars exported */
 extern char *garp_buffer;
@@ -57,6 +56,6 @@ extern int garp_fd;
 /* prototypes */
 extern void gratuitous_arp_init(void);
 extern void gratuitous_arp_close(void);
-extern int send_gratuitous_arp(ip_address *);
+extern int send_gratuitous_arp(ip_address_t *);
 
 #endif
