@@ -36,10 +36,10 @@
 /* Macro definition */
 
 /* prototypes */
-extern void alloc_vrrp_bucket(vrrp_rt *);
-extern void alloc_vrrp_fd_bucket(vrrp_rt *);
-extern void remove_vrrp_fd_bucket(vrrp_rt *);
-extern void set_vrrp_fd_bucket(int, vrrp_rt *);
-extern vrrp_rt *vrrp_index_lookup(const int, const int);
+extern void alloc_vrrp_bucket(vrrp_t *);
+extern void alloc_vrrp_fd_bucket(vrrp_t *);
+extern void remove_vrrp_fd_bucket(vrrp_t *);
+extern void set_vrrp_fd_bucket(int, vrrp_t *);
+extern vrrp_t *vrrp_index_lookup(const int, const int);
 
 #endif
