@@ -21,6 +21,7 @@
  */
 
 /* local include */
+#include "vrrp_ipaddress.h"
 #include "vrrp_iproute.h"
 #include "vrrp_netlink.h"
 #include "vrrp_if.h"
@@ -29,9 +30,44 @@
 #include "memory.h"
 #include "utils.h"
 
+/* Utility functions */
+static int
+add_addr2req(struct nlmsghdr *n, int maxlen, int type, ip_address_t *ip_address)
+{
+	void *addr;
+	int alen;
+
+	if (!ip_address)
+		return -1;
+
+	addr = (IP_IS6(ip_address)) ? (void *) &ip_address->u.sin6_addr :
+				     (void *) &ip_address->u.sin.sin_addr;
+	alen = (IP_IS6(ip_address)) ? sizeof(ip_address->u.sin6_addr) :
+				     sizeof(ip_address->u.sin.sin_addr);
+
+	return addattr_l(n, maxlen, type, addr, alen);
+}
+
+static int
+add_addr2rta(struct rtattr *rta, int maxlen, int type, ip_address_t *ip_address)
+{
+	void *addr;
+	int alen;
+
+	if (!ip_address)
+		return -1;
+
+	addr = (IP_IS6(ip_address)) ? (void *) &ip_address->u.sin6_addr :
+				     (void *) &ip_address->u.sin.sin_addr;
+	alen = (IP_IS6(ip_address)) ? sizeof(ip_address->u.sin6_addr) :
+				     sizeof(ip_address->u.sin.sin_addr);
+
+	return rta_addattr_l(rta, maxlen, type, addr, alen);
+}
+
 /* Add/Delete IP route to/from a specific interface */
 int
-netlink_route_ipv4(ip_route *iproute, int cmd)
+netlink_route(ip_route_t *iproute, int cmd)
 {
 	int status = 1;
 	struct {
@@ -49,7 +85,7 @@ netlink_route_ipv4(ip_route *iproute, int cmd)
 	req.n.nlmsg_len   = NLMSG_LENGTH(sizeof(struct rtmsg));
 	req.n.nlmsg_flags = NLM_F_REQUEST | NLM_F_CREATE;
 	req.n.nlmsg_type  = cmd ? RTM_NEWROUTE : RTM_DELROUTE;
-	req.r.rtm_family  = AF_INET;
+	req.r.rtm_family  = IP_FAMILY(iproute->dst);;
 	req.r.rtm_table   = iproute->table ? iproute->table : RT_TABLE_MAIN;
 	req.r.rtm_scope   = RT_SCOPE_NOWHERE;
 
@@ -63,9 +99,9 @@ netlink_route_ipv4(ip_route *iproute, int cmd)
 
 	/* Set routing entry */
 	req.r.rtm_dst_len = iproute->dmask;
-	addattr_l(&req.n, sizeof(req), RTA_DST,		&iproute->dst, 4);
+	add_addr2req(&req.n, sizeof(req), RTA_DST, iproute->dst);
 	if ((!iproute->blackhole) && (!iproute->gw2))
-		addattr_l(&req.n, sizeof(req), RTA_GATEWAY, &iproute->gw,  4);
+		add_addr2req(&req.n, sizeof(req), RTA_GATEWAY, iproute->gw);
 	if (iproute->gw2) {
 		rta->rta_type = RTA_MULTIPATH;
 		rta->rta_len = RTA_LENGTH(0);
@@ -75,17 +111,17 @@ netlink_route_ipv4(ip_route *iproute, int cmd)
 	rtnh->rtnh_len = sizeof(*rtnh); \
 	if (iproute->index) rtnh->rtnh_ifindex = iproute->index; \
 	rta->rta_len += rtnh->rtnh_len;	\
-	rta_addattr_l(rta, 1024, RTA_GATEWAY, x, 4); \
-	rtnh->rtnh_len += sizeof(struct rtattr) + 4; \
+	add_addr2rta(rta, 1024, RTA_GATEWAY, x); \
+	rtnh->rtnh_len += sizeof(struct rtattr) + IP_SIZE(x); \
 	rtnh = RTNH_NEXT(rtnh);
-		MULTIPATH_ADD_GW(&iproute->gw);
-		MULTIPATH_ADD_GW(&iproute->gw2);
+		MULTIPATH_ADD_GW(iproute->gw);
+		MULTIPATH_ADD_GW(iproute->gw2);
 		addattr_l(&req.n, sizeof(req), RTA_MULTIPATH, RTA_DATA(rta), RTA_PAYLOAD(rta));
 	}
 	if ((iproute->index) && (!iproute->gw2))
 		addattr32(&req.n, sizeof(req), RTA_OIF, iproute->index);
 	if (iproute->src)
-		addattr_l(&req.n, sizeof(req), RTA_PREFSRC, &iproute->src, 4);
+		add_addr2req(&req.n, sizeof(req), RTA_PREFSRC, iproute->src);
 	if (iproute->metric)
 		addattr32(&req.n, sizeof(req), RTA_PRIORITY, iproute->metric);
 
@@ -96,9 +132,9 @@ netlink_route_ipv4(ip_route *iproute, int cmd)
 
 /* Add/Delete a list of IP routes */
 void
-netlink_rtlist_ipv4(list rt_list, int cmd)
+netlink_rtlist(list rt_list, int cmd)
 {
-	ip_route *iproute;
+	ip_route_t *iproute;
 	element e;
 
 	/* No routes to add */
@@ -109,7 +145,7 @@ netlink_rtlist_ipv4(list rt_list, int cmd)
 		iproute = ELEMENT_DATA(e);
 		if ((cmd && !iproute->set) ||
 		    (!cmd && iproute->set)) {
-			if (netlink_route_ipv4(iproute, cmd) > 0)
+			if (netlink_route(iproute, cmd) > 0)
 				iproute->set = (cmd) ? 1 : 0;
 			else
 				iproute->set = 0;
@@ -126,46 +162,55 @@ free_iproute(void *rt_data)
 void
 dump_iproute(void *rt_data)
 {
-	ip_route *route = rt_data;
-	char *log_msg = MALLOC(150);
-	char *tmp = MALLOC(30);
+	ip_route_t *route = rt_data;
+	char *log_msg = MALLOC(1024);
+	char *tmp = MALLOC(INET6_ADDRSTRLEN + 30);
+	char *tmp_str;
 
 	if (route->blackhole) {
 		strncat(log_msg, "blackhole ", 30);
 	}
 	if (route->dst) {
-		snprintf(tmp, 30, "%s/%d", inet_ntop2(route->dst), route->dmask);
-		strncat(log_msg, tmp, 30);
+		tmp_str = ipaddresstos(route->dst);
+		snprintf(tmp, INET6_ADDRSTRLEN + 30, "%s/%d", tmp_str, route->dmask);
+		strncat(log_msg, tmp, INET6_ADDRSTRLEN + 30);
+		FREE(tmp_str);
 	}
 	if (route->gw) {
-		snprintf(tmp, 30, " gw %s", inet_ntop2(route->gw));
-		strncat(log_msg, tmp, 30);
+		tmp_str = ipaddresstos(route->gw);
+		snprintf(tmp, INET6_ADDRSTRLEN + 30, " gw %s", tmp_str);
+		strncat(log_msg, tmp, INET6_ADDRSTRLEN + 30);
+		FREE(tmp_str);
 	}
 	if (route->gw2) {
-		snprintf(tmp, 30, " or gw %s", inet_ntop2(route->gw2));
-		strncat(log_msg, tmp, 30);
+		tmp_str = ipaddresstos(route->gw2);
+		snprintf(tmp, INET6_ADDRSTRLEN + 30, " or gw %s", tmp_str);
+		strncat(log_msg, tmp, INET6_ADDRSTRLEN + 30);
+		FREE(tmp_str);
 	}
 	if (route->src) {
-		snprintf(tmp, 30, " src %s", inet_ntop2(route->src));
-		strncat(log_msg, tmp, 30);
+		tmp_str = ipaddresstos(route->src);
+		snprintf(tmp, INET6_ADDRSTRLEN + 30, " src %s", tmp_str);
+		strncat(log_msg, tmp, INET6_ADDRSTRLEN + 30);
+		FREE(tmp_str);
 	}
 	if (route->index) {
-		snprintf(tmp, 30, " dev %s",
+		snprintf(tmp, INET6_ADDRSTRLEN + 30, " dev %s",
 			 IF_NAME(if_get_by_ifindex(route->index)));
-		strncat(log_msg, tmp, 30);
+		strncat(log_msg, tmp, INET6_ADDRSTRLEN + 30);
 	}
 	if (route->table) {
-		snprintf(tmp, 30, " table %d", route->table);
-		strncat(log_msg, tmp, 30);
+		snprintf(tmp, INET6_ADDRSTRLEN + 30, " table %d", route->table);
+		strncat(log_msg, tmp, INET6_ADDRSTRLEN + 30);
 	}
 	if (route->scope) {
-		snprintf(tmp, 30, " scope %s",
+		snprintf(tmp, INET6_ADDRSTRLEN + 30, " scope %s",
 			 netlink_scope_n2a(route->scope));
-		strncat(log_msg, tmp, 30);
+		strncat(log_msg, tmp, INET6_ADDRSTRLEN + 30);
 	}
 	if (route->metric) {
-		snprintf(tmp, 30, " metric %d", route->metric);
-		strncat(log_msg, tmp, 30);
+		snprintf(tmp, INET6_ADDRSTRLEN + 30, " metric %d", route->metric);
+		strncat(log_msg, tmp, INET6_ADDRSTRLEN + 30);
 	}
 
 	log_message(LOG_INFO, "     %s", log_msg);
@@ -176,13 +221,12 @@ dump_iproute(void *rt_data)
 void
 alloc_route(list rt_list, vector_t *strvec)
 {
-	ip_route *new;
-	uint32_t ipaddr = 0;
-	interface *ifp;
+	ip_route_t *new;
+	interface_t *ifp;
 	char *str;
 	int i = 0;
 
-	new = (ip_route *) MALLOC(sizeof(ip_route));
+	new = (ip_route_t *) MALLOC(sizeof(ip_route_t));
 
 	/* FMT parse */
 	while (i < vector_size(strvec)) {
@@ -191,19 +235,19 @@ alloc_route(list rt_list, vector_t *strvec)
 		/* cmd parsing */
 		if (!strcmp(str, "blackhole")) {
 			new->blackhole = 1;
-			inet_ston(vector_slot(strvec, ++i), &new->dst);
-			new->dmask = inet_stom(vector_slot(strvec, i));
+			new->dst = parse_ipaddress(NULL, vector_slot(strvec, ++i));
+			new->dmask = new->dst->ifa.ifa_prefixlen;
 		} else if (!strcmp(str, "via") || !strcmp(str, "gw")) {
-			inet_ston(vector_slot(strvec, ++i), &new->gw);
+			new->gw = parse_ipaddress(NULL, vector_slot(strvec, ++i));
 		} else if (!strcmp(str, "or")) {
-			inet_ston(vector_slot(strvec, ++i), &new->gw2);
+			new->gw2 = parse_ipaddress(NULL, vector_slot(strvec, ++i));
 		} else if (!strcmp(str, "src")) {
-			inet_ston(vector_slot(strvec, ++i), &new->src);
+			new->src = parse_ipaddress(NULL, vector_slot(strvec, ++i));
 		} else if (!strcmp(str, "dev") || !strcmp(str, "oif")) {
 			ifp = if_get_by_ifname(vector_slot(strvec, ++i));
 			if (!ifp) {
 				log_message(LOG_INFO, "VRRP is trying to assign VROUTE to unknown "
-				       "%s interface !!! go out and fixe your conf !!!",
+				       "%s interface !!! go out and fix your conf !!!",
 				       (char *)vector_slot(strvec, i));
 				FREE(new);
 				return;
@@ -217,9 +261,10 @@ alloc_route(list rt_list, vector_t *strvec)
 			new->scope = netlink_scope_a2n(vector_slot(strvec, ++i));
 		} else {
 			if (!strcmp(str, "to")) i++;
-			if (inet_ston(vector_slot(strvec, i), &ipaddr)) {
-				inet_ston(vector_slot(strvec, i), &new->dst);
-				new->dmask = inet_stom(vector_slot(strvec, i));
+
+			new->dst = parse_ipaddress(NULL, vector_slot(strvec, i));
+			if (new->dst) {
+				new->dmask = new->dst->ifa.ifa_prefixlen;
 			}
 		}
 		i++;
@@ -230,9 +275,9 @@ alloc_route(list rt_list, vector_t *strvec)
 
 /* Try to find a route in a list */
 int
-route_exist(list l, ip_route *iproute)
+route_exist(list l, ip_route_t *iproute)
 {
-	ip_route *ipr;
+	ip_route_t *ipr;
 	element e;
 
 	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
@@ -249,7 +294,8 @@ route_exist(list l, ip_route *iproute)
 void
 clear_diff_routes(list l, list n)
 {
-	ip_route *iproute;
+	ip_route_t *iproute;
+	char *tmp_str;
 	element e;
 
 	/* No route in previous conf */
@@ -259,16 +305,18 @@ clear_diff_routes(list l, list n)
 	/* All Static routes removed */
 	if (LIST_ISEMPTY(n)) {
 		log_message(LOG_INFO, "Removing a VirtualRoute block");
-		netlink_rtlist_ipv4(l, IPROUTE_DEL);
+		netlink_rtlist(l, IPROUTE_DEL);
 		return;
 	}
 
 	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
 		iproute = ELEMENT_DATA(e);
 		if (!route_exist(n, iproute) && iproute->set) {
+			tmp_str = ipaddresstos(iproute->dst);
 			log_message(LOG_INFO, "ip route %s/%d ... , no longer exist"
-			       , inet_ntop2(iproute->dst), iproute->dmask);
-			netlink_route_ipv4(iproute, IPROUTE_DEL);
+					    , tmp_str, iproute->dmask);
+			FREE(tmp_str);
+			netlink_route(iproute, IPROUTE_DEL);
 		}
 	}
 }
