@@ -34,6 +34,7 @@
 #include "logger.h"
 #include "parser.h"
 #include "memory.h"
+#include "bitops.h"
 
 /* Static addresses handler */
 static void
@@ -110,9 +111,24 @@ static void
 vrrp_vmac_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->vmac_flags |= VRRP_VMAC_FL_SET;
-	if (!vrrp->saddr.ss_family && vrrp->family == AF_INET)
-		inet_ip4tosockaddr(IF_ADDR(vrrp->ifp), &vrrp->saddr);
+	interface_t *ifp = vrrp->ifp;
+	struct sockaddr_storage *saddr = &vrrp->saddr;
+
+	__set_bit(VRRP_VMAC_BIT, &vrrp->vmac_flags);
+	if (!vrrp->saddr.ss_family) {
+		if (!ifp) {
+			log_message(LOG_INFO, "Please define interface keyword before use_vmac keyword");
+			return;
+		} else {
+			if (vrrp->family == AF_INET) {
+				inet_ip4tosockaddr(&ifp->sin_addr, saddr);
+			} else if (vrrp->family == AF_INET6) {
+				inet_ip6tosockaddr(&ifp->sin6_addr, saddr);
+				/* IPv6 use-case: Binding to link-local address requires an interface */
+				inet_ip6scopeid(IF_INDEX(ifp), saddr);
+			}
+		}
+	}
 	if (vector_size(strvec) == 2) {
 		strncpy(vrrp->vmac_ifname, vector_slot(strvec, 1),
 			IFNAMSIZ - 1);
@@ -128,8 +144,8 @@ static void
 vrrp_vmac_xmit_base_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	if (vrrp->vmac_flags & VRRP_VMAC_FL_SET)
-		vrrp->vmac_flags |= VRRP_VMAC_FL_XMITBASE;
+	if (__test_bit(VRRP_VMAC_BIT, &vrrp->vmac_flags))
+		__set_bit(VRRP_VMAC_XMITBASE_BIT, &vrrp->vmac_flags);
 }
 static void
 vrrp_unicast_peer_handler(vector_t *strvec)
@@ -142,6 +158,9 @@ vrrp_native_ipv6_handler(vector_t *strvec)
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->family = AF_INET6;
 
+	if (__test_bit(VRRP_VMAC_BIT, &vrrp->vmac_flags))
+		log_message(LOG_INFO, "You should declare native_ipv6 before use_vmac!");
+
 	if (vrrp->auth_type != VRRP_AUTH_NONE)
 		vrrp->auth_type = VRRP_AUTH_NONE;
 }
@@ -174,9 +193,8 @@ vrrp_int_handler(vector_t *strvec)
 		return;
 	}
 
-	if (vrrp->vmac_flags & VRRP_VMAC_FL_SET) {
+	if (__test_bit(VRRP_VMAC_BIT, &vrrp->vmac_flags))
 		netlink_link_add_vmac(vrrp);
-	}
 }
 static void
 vrrp_track_int_handler(vector_t *strvec)
@@ -198,6 +216,7 @@ static void
 vrrp_srcip_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	interface_t *ifp = vrrp->ifp;
 	struct sockaddr_storage *saddr = &vrrp->saddr;
 	int ret;
 
@@ -205,16 +224,26 @@ vrrp_srcip_handler(vector_t *strvec)
 	if (ret < 0) {
 		log_message(LOG_ERR, "Configuration error: VRRP instance[%s] malformed unicast"
 				     " src address[%s]. Skipping..."
-				   , vrrp->iname, vector_slot(strvec, 1));
+				   , vrrp->iname, FMT_STR_VSLOT(strvec, 1));
 		return;
 	}
 
 	if (saddr->ss_family != vrrp->family) {
 		log_message(LOG_ERR, "Configuration error: VRRP instance[%s] and unicast src address"
 				     "[%s] MUST be of the same family !!! Skipping..."
-				   , vrrp->iname, vector_slot(strvec, 1));
+				   , vrrp->iname, FMT_STR_VSLOT(strvec, 1));
 		memset(saddr, 0, sizeof(struct sockaddr_storage));
 	}
+
+	/* IPv6 use-case: Binding to link-local address requires an interface.
+	 * Just specify scope_id for all address types */
+	if (saddr->ss_family == AF_INET6) {
+		if (!ifp) {
+			log_message(LOG_INFO, "Please define interface keyword before mcast_src_ip keyword");
+			return;
+		}
+		inet_ip6scopeid(IF_INDEX(ifp), saddr);
+	}
 }
 static void
 vrrp_vrid_handler(vector_t *strvec)
@@ -228,7 +257,7 @@ vrrp_vrid_handler(vector_t *strvec)
 		       "             must be between 1 & 255. reconfigure !");
 	} else {
 		alloc_vrrp_bucket(vrrp);
-		if (vrrp->vmac_flags & VRRP_VMAC_FL_SET) {
+		if (__test_bit(VRRP_VMAC_BIT, &vrrp->vmac_flags)) {
 			if (strlen(vrrp->vmac_ifname) == 0)
 				snprintf(vrrp->vmac_ifname, IFNAMSIZ, "vrrp.%d", vrrp->vrid);
 			netlink_link_add_vmac(vrrp);
@@ -254,16 +283,16 @@ static void
 vrrp_adv_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->adver_int = atoi(vector_slot(strvec, 1));
+	vrrp->adver_int = atof(vector_slot(strvec, 1)) * 100; /* multiply with 100 to get decimal value */
 
+	/* Simple check. Note that using VRRPv2 with 0.01s advert interval will not report an error */
 	if (VRRP_IS_BAD_ADVERT_INT(vrrp->adver_int)) {
 		log_message(LOG_INFO, "VRRP Error : Advert interval not valid !");
-		log_message(LOG_INFO,
-		       "             must be between less than 1sec.");
+		log_message(LOG_INFO, "             must be >=1sec for VRRPv2 or >=0.01sec for VRRPv3.\n");
 		log_message(LOG_INFO, "             Using default value : 1sec");
-		vrrp->adver_int = 1;
+		vrrp->adver_int = 100;
 	}
-	vrrp->adver_int *= TIMER_HZ;
+	vrrp->adver_int *= TIMER_HZ / 100.0;
 }
 static void
 vrrp_debug_handler(vector_t *strvec)
@@ -363,7 +392,23 @@ static void
 vrrp_garp_refresh_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->garp_refresh = atoi(vector_slot(strvec, 1)) * TIMER_HZ;
+	vrrp->garp_refresh.tv_sec = atoi(vector_slot(strvec, 1));
+}
+static void
+vrrp_garp_rep_handler(vector_t *strvec)
+{
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp->garp_rep = atoi(vector_slot(strvec, 1));
+	if (vrrp->garp_rep < 1)
+		vrrp->garp_rep = 1;
+}
+static void
+vrrp_garp_refresh_rep_handler(vector_t *strvec)
+{
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp->garp_refresh_rep = atoi(vector_slot(strvec, 1));
+	if (vrrp->garp_refresh_rep < 1)
+		vrrp->garp_refresh_rep = 1;
 }
 static void
 vrrp_auth_type_handler(vector_t *strvec)
@@ -492,6 +537,28 @@ vrrp_vscript_fall_handler(vector_t *strvec)
 		vscript->fall = 1;
 }
 
+static void
+vrrp_version_handler(vector_t *strvec)
+{
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+
+	uint8_t version = atoi(vector_slot(strvec, 1));
+	if (VRRP_IS_BAD_VERSION(version)) {
+		log_message(LOG_INFO, "VRRP Error : Version not valid !\n");
+		log_message(LOG_INFO, "             must be between either 2 or 3. reconfigure !\n");
+		return;
+	}
+	vrrp->version = version;
+}
+
+static void
+vrrp_accept_handler(vector_t *strvec)
+{
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+
+	vrrp->accept = true;
+}
+
 vector_t *
 vrrp_init_keywords(void)
 {
@@ -524,11 +591,13 @@ vrrp_init_keywords(void)
 	install_keyword("mcast_src_ip", &vrrp_srcip_handler);
 	install_keyword("unicast_src_ip", &vrrp_srcip_handler);
 	install_keyword("virtual_router_id", &vrrp_vrid_handler);
+	install_keyword("version", &vrrp_version_handler);
 	install_keyword("priority", &vrrp_prio_handler);
 	install_keyword("advert_int", &vrrp_adv_handler);
 	install_keyword("virtual_ipaddress", &vrrp_vip_handler);
 	install_keyword("virtual_ipaddress_excluded", &vrrp_evip_handler);
 	install_keyword("virtual_routes", &vrrp_vroutes_handler);
+	install_keyword("accept", &vrrp_accept_handler);
 	install_keyword("preempt", &vrrp_preempt_handler);
 	install_keyword("nopreempt", &vrrp_nopreempt_handler);
 	install_keyword("preempt_delay", &vrrp_preempt_delay_handler);
@@ -542,6 +611,8 @@ vrrp_init_keywords(void)
 	install_keyword("lvs_sync_daemon_interface", &vrrp_lvs_syncd_handler);
 	install_keyword("garp_master_delay", &vrrp_garp_delay_handler);
 	install_keyword("garp_master_refresh", &vrrp_garp_refresh_handler);
+	install_keyword("garp_master_repeat", &vrrp_garp_rep_handler);
+	install_keyword("garp_master_refresh_repeat", &vrrp_garp_refresh_rep_handler);
 	install_keyword("authentication", NULL);
 	install_sublevel();
 	install_keyword("auth_type", &vrrp_auth_type_handler);
