@@ -32,7 +32,7 @@
 
 /* Compute the new instance sands */
 void
-vrrp_init_instance_sands(vrrp_rt * vrrp)
+vrrp_init_instance_sands(vrrp_t * vrrp)
 {
 	set_time_now();
 
@@ -40,20 +40,25 @@ vrrp_init_instance_sands(vrrp_rt * vrrp)
 	    vrrp->state == VRRP_STATE_GOTO_MASTER ||
 	    vrrp->state == VRRP_STATE_GOTO_FAULT  ||
 	    vrrp->wantstate == VRRP_STATE_GOTO_MASTER) {
-		vrrp->sands.tv_sec = time_now.tv_sec + vrrp->adver_int / TIMER_HZ;
- 		vrrp->sands.tv_usec = time_now.tv_usec;
+		vrrp->sands = timer_add_long(time_now, vrrp->adver_int);
 		return;
 	}
 
+	/*
+	 * When in the BACKUP state the expiry timer should be updated to
+	 * time_now plus the Master Down Timer, when a non-preemptable packet is
+	 * received. (When a preemptable packet is received, the wantstate is
+	 * moved to GOTO_MASTER and this condition is caught above).
+	 */
 	if (vrrp->state == VRRP_STATE_BACK || vrrp->state == VRRP_STATE_FAULT)
 		vrrp->sands = timer_add_long(time_now, vrrp->ms_down_timer);
 }
 
 /* Instance name lookup */
-vrrp_rt *
+vrrp_t *
 vrrp_get_instance(char *iname)
 {
-	vrrp_rt *vrrp;
+	vrrp_t *vrrp;
 	list l = vrrp_data->vrrp;
 	element e;
 
@@ -67,9 +72,9 @@ vrrp_get_instance(char *iname)
 
 /* Set instances group pointer */
 void
-vrrp_sync_set_group(vrrp_sgroup *vgroup)
+vrrp_sync_set_group(vrrp_sgroup_t *vgroup)
 {
-	vrrp_rt *vrrp;
+	vrrp_t *vrrp;
 	char *str;
 	int i;
 
@@ -87,9 +92,9 @@ vrrp_sync_set_group(vrrp_sgroup *vgroup)
 
 /* All interface are UP in the same group */
 int
-vrrp_sync_group_up(vrrp_sgroup * vgroup)
+vrrp_sync_group_up(vrrp_sgroup_t * vgroup)
 {
-	vrrp_rt *vrrp;
+	vrrp_t *vrrp;
 	element e;
 	list l = vgroup->index_list;
 	int is_up = 0;
@@ -110,7 +115,7 @@ vrrp_sync_group_up(vrrp_sgroup * vgroup)
 
 /* SMTP alert group notifier */
 void
-vrrp_sync_smtp_notifier(vrrp_sgroup *vgroup)
+vrrp_sync_smtp_notifier(vrrp_sgroup_t *vgroup)
 {
 	if (vgroup->smtp_alert) {
 		if (GROUP_STATE(vgroup) == VRRP_STATE_MAST)
@@ -126,9 +131,9 @@ vrrp_sync_smtp_notifier(vrrp_sgroup *vgroup)
 
 /* Leaving fault state */
 int
-vrrp_sync_leave_fault(vrrp_rt * vrrp)
+vrrp_sync_leave_fault(vrrp_t * vrrp)
 {
-	vrrp_sgroup *vgroup = vrrp->sync;
+	vrrp_sgroup_t *vgroup = vrrp->sync;
 
 	if (vrrp_sync_group_up(vgroup)) {
 		log_message(LOG_INFO, "VRRP_Group(%s) Leaving FAULT state",
@@ -140,10 +145,10 @@ vrrp_sync_leave_fault(vrrp_rt * vrrp)
 
 /* Check transition to master state */
 int
-vrrp_sync_goto_master(vrrp_rt * vrrp)
+vrrp_sync_goto_master(vrrp_t * vrrp)
 {
-	vrrp_rt *isync;
-	vrrp_sgroup *vgroup = vrrp->sync;
+	vrrp_t *isync;
+	vrrp_sgroup_t *vgroup = vrrp->sync;
 	list l = vgroup->index_list;
 	element e;
 
@@ -165,10 +170,10 @@ vrrp_sync_goto_master(vrrp_rt * vrrp)
 }
 
 void
-vrrp_sync_master_election(vrrp_rt * vrrp)
+vrrp_sync_master_election(vrrp_t * vrrp)
 {
-	vrrp_rt *isync;
-	vrrp_sgroup *vgroup = vrrp->sync;
+	vrrp_t *isync;
+	vrrp_sgroup_t *vgroup = vrrp->sync;
 	list l = vgroup->index_list;
 	element e;
 
@@ -195,10 +200,10 @@ vrrp_sync_master_election(vrrp_rt * vrrp)
 }
 
 void
-vrrp_sync_backup(vrrp_rt * vrrp)
+vrrp_sync_backup(vrrp_t * vrrp)
 {
-	vrrp_rt *isync;
-	vrrp_sgroup *vgroup = vrrp->sync;
+	vrrp_t *isync;
+	vrrp_sgroup_t *vgroup = vrrp->sync;
 	list l = vgroup->index_list;
 	element e;
 
@@ -226,10 +231,10 @@ vrrp_sync_backup(vrrp_rt * vrrp)
 }
 
 void
-vrrp_sync_master(vrrp_rt * vrrp)
+vrrp_sync_master(vrrp_t * vrrp)
 {
-	vrrp_rt *isync;
-	vrrp_sgroup *vgroup = vrrp->sync;
+	vrrp_t *isync;
+	vrrp_sgroup_t *vgroup = vrrp->sync;
 	list l = vgroup->index_list;
 	element e;
 
@@ -261,10 +266,10 @@ vrrp_sync_master(vrrp_rt * vrrp)
 }
 
 void
-vrrp_sync_fault(vrrp_rt * vrrp)
+vrrp_sync_fault(vrrp_t * vrrp)
 {
-	vrrp_rt *isync;
-	vrrp_sgroup *vgroup = vrrp->sync;
+	vrrp_t *isync;
+	vrrp_sgroup_t *vgroup = vrrp->sync;
 	list l = vgroup->index_list;
 	element e;
 
